function [pcellKS,hcellKS,Tnew]=KSTestnew(T,M,alpha,xvec,yvec,exponent)

% T is a matrix of size numNodes-by-lkvec, where lkvec is the length of the
% kvector used in the clustering algorithm (i.e., the number of different
% clustering trials). Each column represents the outcome of a single
% clustering analysis, and each entry corresponds to that node's assigned
% cluster ID.
%
% The goal here is, for each column of T, to compare all patch-average CDFs
% against each other using the Kolmogorov-Smirnov test to see whether they
% are significantly different from each other.

[~,c] = size(T);

ny = length(yvec);
nx = length(xvec);

pcellKS = cell(c,1);
hcellKS = cell(c,1);

% Tnewcell = cell(c,1);
Tnew = zeros(size(T));

Mdsfullvec = M.ds_full(:); 

for i=1:c
%     
%     tic
    fprintf('KSTestnew: Starting i = %i of %i\n',i,c)
    
    Ttemp = T(:,i);

    
    % get the grains for input to rank sum
    grainscell = cell(max(Ttemp),1); %will contain vectors of raw grain sizes
    
    for aa = 1:max(Ttemp)
        tempgrains = [];
        inds = find(Ttemp == aa);
        for jj=1:length(inds)
           tempgrains = [tempgrains; Mdsfullvec{inds(jj)}];
        end
        grainscell{aa} = tempgrains; 
%         grainscell{aa} = log2(tempgrains); %need to be in psi units!

    end
    
    % get range of grain sizes, make a closely-spaced vector of grain size
    % classes to interpolate the distributions onto.
    Dmin = 1e10;
    Dmax = -1e10;
    for aa=1:max(Ttemp)
        Dmint = min(grainscell{aa});
        if Dmint < Dmin
            Dmin = Dmint;
        end
        Dmaxt = max(grainscell{aa});
        if Dmaxt > Dmax
            Dmax = Dmaxt;
        end
    end
    psimin = log2(Dmin);
    psimax = log2(Dmax);
    psivec = linspace(psimin,psimax,50);
    psivec = psivec';
%     psivec = [psimin:0.1:psimax]';
    Dvec = 2.^psivec;
    
    % compute cdfs and convert to vol-by-weight
    cdfAbNmat = zeros(length(Dvec),max(Ttemp));
    cdfVbWmat = zeros(length(Dvec),max(Ttemp));
    for aa=1:max(Ttemp)
        [cdf1,D1] = ecdf(grainscell{aa});
        psi1 = log2(D1);
        cdfinterp = zeros(length(Dvec),1);
        for bb=1:length(psivec)-1
%             fprintf('aa=%i, bb=%i\n',aa,bb)
            if max(psi1) <= psivec(bb)
                cdfinterp(bb) = 1;
            else
                indH = min(find(psi1>psivec(bb)));
                indL = max(find(psi1<=psivec(bb)));
                cdfinterp(bb) = (cdf1(indH)-cdf1(indL))/(psi1(indH)-psi1(indL))...
                    * (psivec(bb)-psi1(indL)) + cdf1(indL);
            end
        end
        cdfinterp(length(Dvec)) = 1;
            
        cdfAbNmat(:,aa) = 100.* cdfinterp;
        [cdfVbWmat(:,aa),~] = convertgsd(cdfAbNmat(:,aa),Dvec,exponent);
    end
    
%     figure
%     for aa=1:max(Ttemp)
%         semilogx(Dvec,cdfAbNmat(:,aa),'-k')
%         hold on
%         semilogx(Dvec,cdfVbWmat(:,aa),'-r')
%         pause(1)
%     end

    % compute the KS test statistic (two-tailed)
    KS = zeros(max(Ttemp));
    pValue = zeros(max(Ttemp));
    H = -2 .* ones(max(Ttemp));
    for aa = 1:max(Ttemp)-1
        for bb=aa+1:max(Ttemp)
            
            KS(aa,bb) = max(abs(cdfVbWmat(:,aa)-cdfVbWmat(:,bb)))/100;
            n1 = length(grainscell{aa});
            n2 = length(grainscell{bb});
            n = n1*n2/(n1+n2);
            lambda =  max((sqrt(n) + 0.12 + 0.11/sqrt(n)) * KS(aa,bb) , 0);
            
            j       =  (1:101)';
            pValuetemp  =  2 * sum((-1).^(j-1).*exp(-2*lambda*lambda*j.^2));
            pValue(aa,bb)  =  min(max(pValuetemp, 0), 1);
            
            H(aa,bb)  =  -1*(alpha >= pValuetemp);
            % H = 0 if samples come from same distribution (null hypothesis
            % is not rejected)
            
        end
    end
    
    pcellKS{i} = pValue;
    hcellKS{i} = H;
    
    % consolidate patch types: if two patches are found to be statistically
    % indistinguishable (based on their gsds) give them the same label.
    Tnewtemp = zeros(size(Ttemp));
    
    patchcell = cell(max(Ttemp),1);
    for jj=1:max(Ttemp)
        colinds = find(H(jj,:) == 0);
        if isempty(colinds)
            patchcell{jj} = jj;
        else
            patchcell{jj} = [jj colinds];
        end
    end
    numpatches = 0;
    for jj=1:max(Ttemp)
        inds = patchcell{jj};
        if numpatches == 0
            numpatches = 1;
            newpatchcell{1} = inds;
        else
            unmatched = 1;
            mm=1;
            while unmatched == 1 && mm <= numpatches
                patchindstemp = newpatchcell{mm};
                if max(ismember(inds,patchindstemp)) == 1
                    patchindstemp = [patchindstemp inds];
                    newpatchcell{mm} = unique(patchindstemp);
                    unmatched = 0;
                else
                    mm = mm + 1;
                end
            end
            if unmatched == 1
                numpatches = numpatches + 1;
                newpatchcell{numpatches} = inds;
            end
        end
    end
    
    for jj=1:numpatches
        for kk=1:length(newpatchcell{jj})
            ind = newpatchcell{jj}(kk);
            Tnewtemp(Ttemp==ind) = jj;
        end
    end
    
    Tnew(:,i) = Tnewtemp;
    fprintf('kold: %i; knew: %i\n',max(Ttemp),numpatches);
end
    
            
        
% 
%     % plot patches that are the same, if any
%     if max(max(H)) == 0
%         figure
%         [rr,cc] = find(H==0);
%         k=1;
%         for jj=1:length(rr)
%             dum = zeros(ny,nx);
%             dum = dum(:);
%             inds1 = find(Ttemp == rr(jj));
%             inds2 = find(Ttemp == cc(jj));
%             dum(inds1) = 1;
%             dum(inds2) = 2;
%             subplot(length(rr),1,k)
%             imagesc(xvec,yvec,reshape(dum,ny,nx));
%             axis equal
%             set(gca,'YDir','reverse')
%             ylim([yvec(1)-(yvec(2)-yvec(1))/2 yvec(ny)+(yvec(ny)-yvec(ny-1))/2])
%             xlim([xvec(1)-(xvec(2)-xvec(1))/2 xvec(nx)+(xvec(nx)-xvec(nx-1))/2])
%             title(['i = ' num2str(i) ', rr = ' num2str(rr(jj)) ...
%                 ', cc = ' num2str(cc(jj)) ', pvalue = ' num2str(pValue(rr(jj),cc(jj)))])
%             k=k+1;
%             
%         end
        
%         %plot the cdfs; all of them in black, the two of interest in blue
%         %and red.
%         for jj=1:length(rr)
%             figure
%             for kk=1:max(Ttemp)
%                 semilogx(Dvec,cdfVbWmat(:,kk),'-k')
%                 hold on
%             end
%             plot(Dvec,cdfVbWmat(:,rr(jj)),'-b');
%             hold on
%             plot(Dvec,cdfVbWmat(:,cc(jj)),'-r');
% %             legend('rr','cc')
%             title(['i = ' num2str(i) ', rr = ' num2str(rr(jj)) ...
%                 ' (blue), cc = ' num2str(cc(jj)) ' (red), pvalue = ' ...
%                 num2str(pValue(rr(jj),cc(jj)))])
%         end
%     end
% % 
%     toc
%     
% end