function [D, cdf, D16, D50, D84] = analyzegrains(grains,lowtrunc,exponent)
%
% [D, cdf, D16, D50, D84] = analyzegrains(grains,lowertrunc,exponent)
% 
% reads in grains (output from identifygrains.m)
% [D cdf] is the processed grain size distribution, other outputs are
% standard grain size distribution statistics.
% lowtrunc = lower truncation threshold (mm) to apply. set to 0 if no
% truncation desired.
% exponent is the exponent to be used to convert the area-by-number gsd
% from the image processing to something else. Typical exponents are:
% volume by weight to area by number: -2; to area by weight: 1
% area by number to volume by weight: 2; to area by weight: 3
% area by weight to volume by weight: -1; to area by number: -3
% note that grid-by-number is equivalent to volume-by-weight
% set exponent to 0 if no conversion desired

% generate a cdf of the grains measured by the algorithm (area by number)
[cdf1_AbNtemp, D1temp] = ecdf(grains(:,4));
cdf1_AbNtemp = cdf1_AbNtemp.*100;
%need to go through and remake cdf in case there are repeated grain sizes
jj=1;
for i=1:length(cdf1_AbNtemp)-1
    if D1temp(i+1)-D1temp(i)>=0.000001
       D1(jj) = D1temp(i);
       cdf1_AbN(jj) = cdf1_AbNtemp(i);
       jj = jj+1;
    end
end
if D1temp(length(D1temp))-D1temp(length(D1temp)-1)>=0.000001
    D1(jj) = D1temp(length(D1temp));
    cdf1_AbN(jj) = cdf1_AbNtemp(length(D1temp));
end
cdf_working = cdf1_AbN;
D_working = D1;

% truncate gsd if desired
if lowtrunc~=0
    [cdf1trunc_AbN, D1trunc] = truncategsd(cdf_working,D_working,lowtrunc);
    cdf_working = cdf1trunc_AbN;
    D_working = D1trunc;
end

% convert gsd if desired
if exponent~=0
    [cdfconv, Dconv] = convertgsd(cdf_working,D_working,exponent);
    cdf_working = cdfconv;
    D_working = Dconv;
end

D = D_working';
cdf = cdf_working;


%calculate statistics
D16 = finer(cdf_working,D_working,16);
D50 = finer(cdf_working,D_working,50);
D84 = finer(cdf_working,D_working,84);




%subfunction finer
function out = finer(cdfin,Din,val);
phiin = log2(Din);
phi_interp = interp1(cdfin,phiin,val);
out = 2.^phi_interp;