function A = createAdjacency(nx,ny,nn)
%
% Creates 8-neighbor sparse adjacency matrix for an ny-by-nx grid where
% values of 1 indicate adjacency. 
% A: (nx*ny) by (nx*ny) matrix; row and column indices correspond to
% vectorized locations on the original ny by nx grid.
% added 9/30/2010
% nn = number of neighbors, needs to be 4 or 8.

if nn~=4 && nn~= 8
    error('number of neighbors must be 4 or 8')
end

ivec = [];
jvec = [];
numNodes = nx*ny;
for i=1:ny
    for j=1:nx
        ind = (j-1)*ny + i;
        indR = ind + ny;
        indL = ind - ny;
        indU = ind - 1;
        indD = ind + 1;
        if nn == 8
            indUR = indR - 1; %upper right
            indUL = indL - 1; %upper left
            indLR = indR + 1; %lower right
            indLL = indL + 1; %lower left
        end
        
        % make sure the edges aren't screwed up
        if mod(ind,ny) == 1, indU = []; 
            if nn==8 indUL = []; indUR = []; end 
        end
        if mod(ind,ny) == 0, indD = []; 
            if nn==8 indLR = []; indLL = []; end 
        end
        if indR > numNodes, indR = []; 
            if nn==8 indUR = []; indLR = []; end 
        end
        if indL < 1, indL = []; 
            if nn==8 indUL = []; indLL = []; end 
        end
        
        if nn == 8
            indscol = [indR;indL;indU;indD;indUR;indUL;indLR;indLL];
        else
            indscol = [indR;indL;indU;indD];
        end
        ninds = length(indscol);
        indcol = ones(ninds,1).*ind;
        
        ivec = [ivec; indcol];
        jvec = [jvec; indscol];
    end
end

svec = ones(size(ivec));

A = sparse(ivec,jvec,svec,numNodes,numNodes);