function [L,vgsd] = createLinkage(vgsd_orig,A,metric)
% vgsd is updated vgsd with new (averaged) gsds of created clusters
% metric is minkowski exponent: 1 = cityblock, 2 = euclidean, ...

[numNodes,numD] = size(vgsd_orig);

fprintf(1,'Constructing linkage tree\n');
h = waitbar(0,'Constructing linkage tree');
tic

% this will count the number of cells in each cluster. with agglomerative
% clustering there will be a total of 2*numNodes - 1 clusters.
Area = zeros(2*numNodes -1, 1);
Area(1:numNodes) = 1;

%set up L, equivalent to what comes out of matlab's linkage function
L = zeros(numNodes-1, 3);

%allocate memory for vgsd, which will be updated at each clustering
%iteration and used to compute distances between clusters.
vgsd = zeros(2*numNodes - 1,numD);
vgsd(1:numNodes,:) = vgsd_orig;

% big for loop to construct Ll.;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

for j=1:numNodes-1
    
    % compute distance between pdfs
    if metric == 1
        distGSD = squareform(pdist(vgsd,'cityblock'));
    elseif metric == 2
        distGSD = squareform(pdist(vgsd,'euclidean'));
    end
%     distPDFvec = pdist(vecpdf);
%     Aind = find(A2 == 1);
%     Dvec = distPDFvec(Aind);
%     Xij = Aind(find(Dvec == min(Dvec)));
%     rowij = rowinds(find(zzz==Xij));
%     colij = colinds(find(zzz==Xij));
    
    % find smallest distance with A = 1 (adjacent)
    [Ai,Aj] = find(A == 1);
    Aij = sub2ind(size(distGSD),Ai,Aj);
    Dvec = distGSD(Aij);
    Xij = Aij(find(Dvec == min(Dvec)));
    % Xij are the indices in distPDF where both A=1 and distPDF
    % is the minimum value from the population where A=1.
   
    % just take the first match. binary tree will not allow the
    % same node to be linked to more than one other node, so if one node is
    % linked to two others with the same distPDF, the first two will
    % combine, then be combined with the third node in the next iteration.
    [xi,xj] = ind2sub(size(distGSD),Xij(1));
    

    % update A
    % add new row and column for new cluster
    % set cluster components rows and columns to 0 so they are ignored in
    % the future - this might be able to be done outside the inner for
    % loop
    A(numNodes + j,:) = max(A([xi xj],:));
    A(:,numNodes + j) = max(A([xi xj],:));
    A(xi,:) = 0; A(xj,:) = 0; A(:,xi) = 0; A(:,xj) = 0;

    % add area-averaged gsd to vgsd (should work for both pdf and cdf gsds
    vgsd(numNodes + j,:) = (vgsd(xi,:).*Area(xi) + ...
        vgsd(xj,:).*Area(xj))./(Area(xi) + Area(xj));

    % add to Area vector
    Area(numNodes + j) = Area(xi) + Area(xj);

    % Update linkage matrix
    L(j,1) = xi;
    L(j,2) = xj;
    L(j,3) = min(Dvec);

    % clear old variables
    clear distGSD xi xj Ai Aj Aij Dvec Xij
    
    waitbar(j/(numNodes-1))
        
    
end
toc
close(h)