function [L, vgsd, rank] = createLinkage2(vgsd_orig,A2,metric)
% much faster than createLinkage
% just read in original A, unlike original createLinkage function.
%
% A2 = [numNodes by numNodes] adjacency matrix
% metric: used to compute distance between gsds
%           1: cityblock
%           2: euclidean
%           other: minkowski, with that value as the exponent

[numNodes,numD] = size(vgsd_orig);

fprintf(1,'Constructing linkage tree\n');
h = waitbar(0,'Constructing linkage tree');
tic

% make new adjacency matrix for agglomerative clustering
totclusters = 2*numNodes - 1;
A = zeros(totclusters);
A(1:numNodes,1:numNodes) = A2;

% rank vector: shows the number of merges necessary to reach that cluster
% in a tree (leaves are rank 0)
rank = zeros(totclusters,1);

% this will count the number of cells in each cluster. with agglomerative
% clustering there will be a total of 2*numNodes - 1 clusters.
Area = zeros(2*numNodes -1, 1);
Area(1:numNodes) = 1;

%set up L, equivalent to what comes out of matlab's linkage function
L = zeros(numNodes-1, 3);

%allocate memory for vgsd, which will be updated at each clustering
%iteration and used to compute distances between clusters.
vgsd = zeros(2*numNodes - 1,numD);
vgsd(1:numNodes,:) = vgsd_orig;

%allocate memory for final distGSD, and fill in the first part using pdist
distGSD = zeros(2*numNodes - 1);
distGSD(1:numNodes,1:numNodes) = squareform(pdist(vgsd_orig,'minkowski',metric));



for j=1:numNodes - 1
    
    % A gets updated each iteration, so each time we need to find the
    % nonzero (adjacent) nodes. Then check the (updated) distGSD matrix and
    % pull the two closest nodes.
    
    [r,c,v] = find(A); %nonzero elements
    Ainds = sub2ind(size(distGSD),r,c);
    
    [C,I] = min(distGSD(Ainds));
    xi = r(I); %cluster component 1
    xj = c(I); %cluster component 2
    
    % update L
    L(j,1) = xi;
    L(j,2) = xj;
    L(j,3) = C; %distance, could just use j if only care about order.
    
    % update A
    A(numNodes + j,:) = A(xi,:) | A(xj,:); %max(A([xi xj],:));
    A(:,numNodes + j) = A(xi,:) | A(xj,:); %max(A([xi xj],:));
    A(xi,:) = 0; A(xj,:) = 0; A(:,xi) = 0; A(:,xj) = 0;
    
    % update rank
    rank(numNodes+j) = max([rank(xi) rank(xj)]) + 1;
    
    % add area-averaged gsd to vgsd (should work for both pdf and cdf gsds
    newvgsd = (vgsd(xi,:).*Area(xi) + vgsd(xj,:).*Area(xj))./(Area(xi) + Area(xj));
    
    vgsd(numNodes + j,:) = newvgsd;
    
    % add to Area vector
    Area(numNodes + j) = Area(xi) + Area(xj);
    
    % update distGSD
    vmat = [repmat(newvgsd,numNodes+j,1); zeros(numNodes-j-1,numD)];
    % use minkowski with metric...
    dvec = (sum(abs(vgsd - vmat).^metric,2)).^(1/metric);
    distGSD(numNodes+j,:) = dvec';
    distGSD(:,numNodes+j) = dvec;
    
    clear xi xj dvec newvgsd r c v C I distGSDtemp
    
    waitbar(j/(numNodes-1))
end

toc
close(h)
