function W = createSimilarity(vgsd,M,r,sig1,sig2,metric,dx,dy)
%
% W = sparse similarity matrix computed as a combination of Gaussian
%   functions; one (scaled by sig1) that uses the distance between the gsd
%   vectors and one that uses the physical distance between nodes (scaled
%   by sig2). nodes whose physical separation is greater than r are
%   assigned a similiarity of 0.
% metric is the factor used in a minkowski distance function on the grain
%   size distributions. metric = 1 is cityblock, metric = 2 is euclidean.

if nargin == 5
    metric = 2; %euclidean
end

% compute distXY, physical distance between nodes
distXY = squareform(pdist([M.x(:) M.y(:)]));

% try switching distXY to be in pixel units
distXY = distXY./sqrt(dx^2 + dy^2);

% compute distGSD, euclidean distance between grain size variables
distGSD = squareform(pdist(vgsd,'minkowski',metric));

% normalize distGSD to be between 0 and 1
distGSD = distGSD./max(max(distGSD));

W = (exp(-distGSD.^2./sig1) .* exp(-distXY./sig2));
W(distXY >= r) = 0;
W = sparse(W);

