function IDX = kmeansgsd(M,xvec,yvec,vecgsd,k,gsdfig,mapfig,D50color,cmap,edgecol)
%
% clusters the gsd curves into k clusters using the k-means algorithm, then
% plots the clustered cdfs and maps the clusters.
%
% vecgsd should be in the format of either vecpdf or veccdf
%
% gsdfig and mapfig: 0 for no figures, 1 for figures
% D50color: use color scheme based on D50.

[r,c] = size(vecgsd);
[ny,nx,nd] = size(M.pdf);

if c == nd
    xplot = M.di; %pdf
elseif c == nd+1
    xplot = M.ds; %cdf
end

fontsize = 12;


IDX = kmeans(vecgsd,k,'replicates',10);

if D50color ~=1

    if mapfig == 1
        figure
        imagesc(xvec,yvec,reshape(IDX,ny,nx))
    %     pcolor(M.x,M.y,reshape(IDX,ny,nx))
        shading flat
        axis equal
        title(strcat('k = ',num2str(i)));
        set(gca,'YDir','reverse')
        title(['kmeans clustered gsds, k = ' num2str(k)])
        xlim([min(xvec) max(xvec)]);
        ylim([min(yvec) max(yvec)]);

        set(gca,'TickDir','out')
    end

    if gsdfig == 1
        figure
        colorramp = {'k' 'b' 'c' 'm' 'g' 'y' 'r'};
        for j=1:k
            colorind = rem(j,7);
            if colorind == 0
                colorind = 7;
            end
            ind = find(IDX == j);
            plot(xplot,vecgsd(ind,:),['-' colorramp{colorind}])
            hold on
            set(gca,'XScale','log')
            xlim([min(M.ds) max(M.ds)])
            ylim([0 max(max(vecgsd))])
            title(['kmeans clustered gsds, k = ' num2str(k)])
            pause(1)
        end
    end
    
else
    
    if mapfig == 1
        kvec = k;
        titstr = ['kmeans clustered gsds, k = ' num2str(k)];
        mapD50color(IDX,vecgsd,xplot,xvec,yvec,kvec,titstr,cmap,edgecol)
    end
    
    % give gsds same colorscheme as map
    if gsdfig == 1
        
        %determine D50 for each cluster
        T50vec = zeros(max(IDX),1);
        for aa = 1:max(IDX)
            inds = find(IDX == aa);
            temp = vecgsd(inds,:);
            temp = mean(temp,1); % avg cdf of patch aa
            T50vec(aa) = calcDx(M.ds,temp,50);
        end
        
        minT50 = min(T50vec);
        maxT50 = max(T50vec);
        
        c = colormap(cmap);
        [rr,cc] = size(c);
        dd=linspace(0,1,rr);
        
        figure
        
        for j=1:max(IDX)
            ind = find(IDX == j);
            h=plot(xplot,vecgsd(ind,:),'-');
            
            tempcolor = (T50vec(j)-minT50)/(maxT50-minT50); %0 to 1 scale
            tempcolorind = round(interp1(dd,1:rr,tempcolor));
            
            set(h,'Color',c(tempcolorind,:));
            hold on
            set(gca,'XScale','log')
            xlim([min(M.ds) max(M.ds)])
            ylim([0 max(max(vecgsd))])
            xlabel('D (mm)','FontSize',fontsize);
            ylabel('Percent finer','FontSize',fontsize);
            set(gca,'FontSize',fontsize);
            title(['kmeans clustered gsds, k = ' num2str(k)],'FontSize',1.25*fontsize)
            pause(1)
        end
        
    end
end
