function [KScell,pvalcell,Hcell,T50cell]=kolmogorovTest(T,M,vcdf,ds,alpha,figureon)

% T is a matrix of size numNodes-by-lkvec, where lkvec is the length of the
% kvector used in the clustering algorithm (i.e., the number of different
% clustering trials). Each column represents the outcome of a single
% clustering analysis, and each entry corresponds to that node's assigned
% cluster ID.
%
% The goal here is, for each column of T, to compare all patch-average CDFs
% against each other using the Kolmogorov-Smirnov test to see whether they
% are significantly different from each other.

[~,c] = size(T);

KScell = cell(c,1);
pvalcell = cell(c,1);
Hcell = cell(c,1);
T50cell = cell(c,1);

for i=1:c
    
    Ttemp = T(:,i);
    [~,nd] = size(vcdf);
    avgcdfs = zeros(max(Ttemp),nd);
    nmat = zeros(max(Ttemp),1);
%     avgpdfs = zeros(max(Ttemp),nd-1);
    T50vec = zeros(max(Ttemp),1);
    
    if(figureon)
        figure
    end
    
    % generate the average cdf for each patch type
    for aa = 1:max(Ttemp)
        inds = find(Ttemp == aa);
        temp = vcdf(inds,:);
        avgcdfs(aa,:) = mean(temp,1); % avg cdf of patch aa
        
%         [~,pdft] = cdf2pdf(ds,avgcdfs(aa,:));
%         avgpdfs(aa,:) = pdft;
        
        numgrains = M.numGrains(:);
        nmat(aa) = sum(numgrains(inds));
        
        % list of D50s
        T50vec(aa) = calcDx(ds,avgcdfs(aa,:),50);
        
        % plot cdfs
        if(figureon)
            semilogx(ds,avgcdfs(aa,:))
            hold on
        end
    end
    
    if(figureon)
        title(['k = ', num2str(max(Ttemp))])
    end
    
    
    
    
    
    % adapt kstest2.m for our purposes - we already have the cdfs so this
    % should be faster and easier.
    
    % compute KS statistic
    % KS is a matrix where entry (i,j) is the KS statistic comparing patch
    %  i with patch j.
    % pvalue is a matrix of the same size with the corresponding p-values.
    % H is a matrix of the same size with values of 1 if the null
    %  hypothesis that patch type i and j come from the same distribution
    %  can be rejected.
    KS = zeros(max(Ttemp));
    pValue = zeros(max(Ttemp));
    H = zeros(max(Ttemp));
    for aa = 1:max(Ttemp)-1
        for bb=aa+1:max(Ttemp)
            
%             x1 = avgpdfs(aa,:);
%             x2 = avgpdfs(bb,:);
%             tail = 'unequal';
            
%             [Ht, pV, KSstatistic] = kstest2(x1, x2, alpha, tail);
            
%             KS(aa,bb) = KSstatistic;
%             pValue(aa,bb) = pV;
%             H(aa,bb) = Ht;
            
            KS(aa,bb) = max(abs(avgcdfs(aa,:)-avgcdfs(bb,:)))/100;
            n = (nmat(aa)*nmat(bb))/(nmat(aa)+nmat(bb));
%             n = (nd-1)^2/(2*(nd-1));
            lambda =  max((sqrt(n) + 0.12 + 0.11/sqrt(n)) * KS(aa,bb) , 0);
            
            j       =  (1:101)';
            pValuetemp  =  2 * sum((-1).^(j-1).*exp(-2*lambda*lambda*j.^2));
            pValue(aa,bb)  =  min(max(pValuetemp, 0), 1);
            
            H(aa,bb)  =  (alpha >= pValuetemp);
        end
    end
    
    KScell{i} = KS;
    pvalcell{i} = pValue;
    Hcell{i} = H;
    T50cell{i} = T50vec;
    
    
end