function mapD50color(T,vcdf,ds,xvec,yvec,kvec,titstr,cmap,edgecol)
%
% plots patch maps from already classified matrices
%
% inputs:
%   T: numNodes-by-length(kvec) matrix where columns are the patch type of
%       each node. T is reshaped into an ny-by-nx matrix prior to plotting
%   vcdf: numNodes-by-nd matrix where each row is a cdf
%   ds: grain size vector corresponding to cdfs in vcdf
%   xvec, yvec: vectors of x and y coordinates
%   kvec: vector of k values for clustering, corresponding to T.
%   titstr: title string (e.g., 'Agglomerative')


fontsize = 12;

ny = length(yvec);
nx = length(xvec);
[numNodes,nd] = size(vcdf);

% spatial coordinate grids
[XI,YI] = meshgrid(xvec,yvec);
XIL = XI-((XI-circshift(XI,[0 1]))./2);
XIL(:,1) = XI(:,1)-(XI(:,2)-XI(:,1))./2;
XIR = XI+((circshift(XI,[0 -1])-XI)./2);
XIR(:,nx) = XI(:,nx)+(XI(:,nx)-XI(:,nx-1))./2;
YIT = YI-((YI-circshift(YI,[1 0]))./2);
YIT(1,:) = YI(1,:)-((YI(2,:)-YI(1,:))./2);
YIB = YI+((circshift(YI,[-1 0])-YI)./2);
YIB(ny,:) = YI(ny,:)+((YI(ny,:)-YI(ny-1,:))./2);


% set up T50, equivalent to T but with D50 values instead of integers for
% each patch.
T50 = zeros(size(T));

for i=1:length(kvec)
    Ttemp = T(:,i);
%     Ttemp = T(:,kvec(i)-1);

    %determine D50 for each cluster
    T50vec = zeros(max(Ttemp),1);
    for aa = 1:max(Ttemp)
        inds = find(Ttemp == aa);
        temp = vcdf(inds,:);
        temp = mean(temp,1); % avg cdf of patch aa
        T50vec(aa) = calcDx(ds,temp,50);
        T50(inds,i) = T50vec(aa);
    end
    if i==1
        minT50 = min(T50vec);
        maxT50 = max(T50vec);
    else
        if min(T50vec) < minT50
            minT50 = min(T50vec);
        end
        if max(T50vec) > maxT50
            maxT50 = max(T50vec);
        end
    end
end

%now do the plotting

figure;
jj=1;
for i=1:length(kvec)
    k = kvec(i);
    if length(kvec) > 1
        subplot(ceil(length(kvec)/2),2,jj);
    end

    imagesc(xvec,yvec,reshape(T50(:,i),ny,nx))
    axis equal
    title(['k = ',num2str(k)],'FontSize',fontsize);
    set(gca,'YDir','reverse')
    set(gcf,'Position',[80 156 1068 840]);
    colormap(cmap)
    ylim([yvec(1)-(yvec(2)-yvec(1))/2 yvec(ny)+(yvec(ny)-yvec(ny-1))/2])
    xlim([xvec(1)-(xvec(2)-xvec(1))/2 xvec(nx)+(xvec(nx)-xvec(nx-1))/2])
    caxis([minT50 maxT50])

    set(gca,'TickDir','out','FontSize',fontsize,'YTick',[1 2])
    hold on

    % outline each patch

    TT = reshape(T(:,i),ny,nx);
    perim = zeros(2);
    for bi=1:ny
        for bj=1:nx
            %top
            if bi==1
                perim(1,1) = XIL(bi,bj);
                perim(1,2) = YIT(bi,bj);
                perim(2,1) = XIR(bi,bj);
                perim(2,2) = YIT(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            elseif TT(bi-1,bj) ~= TT(bi,bj)
                perim(1,1) = XIL(bi,bj);
                perim(1,2) = YIT(bi,bj);
                perim(2,1) = XIR(bi,bj);
                perim(2,2) = YIT(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            end
            %right
            if bj == nx
                perim(1,1) = XIR(bi,bj);
                perim(1,2) = YIT(bi,bj);
                perim(2,1) = XIR(bi,bj);
                perim(2,2) = YIB(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            elseif TT(bi,bj+1) ~= TT(bi,bj)
                perim(1,1) = XIR(bi,bj);
                perim(1,2) = YIT(bi,bj);
                perim(2,1) = XIR(bi,bj);
                perim(2,2) = YIB(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            end
            %bottom
            if bi == ny
                perim(1,1) = XIR(bi,bj);
                perim(1,2) = YIB(bi,bj);
                perim(2,1) = XIL(bi,bj);
                perim(2,2) = YIB(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            elseif TT(bi+1,bj) ~= TT(bi,bj)
                perim(1,1) = XIR(bi,bj);
                perim(1,2) = YIB(bi,bj);
                perim(2,1) = XIL(bi,bj);
                perim(2,2) = YIB(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            end
            %left
            if bj == 1
                perim(1,1) = XIL(bi,bj);
                perim(1,2) = YIB(bi,bj);
                perim(2,1) = XIL(bi,bj);
                perim(2,2) = YIT(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            elseif TT(bi,bj-1) ~= TT(bi,bj)
                perim(1,1) = XIL(bi,bj);
                perim(1,2) = YIB(bi,bj);
                perim(2,1) = XIL(bi,bj);
                perim(2,2) = YIT(bi,bj);
                plot(perim(:,1),perim(:,2),['-' edgecol])
                perim = zeros(2);
            end
        end
    end
    
    % find position locations for colorbar axes
    if i==1
        pos = get(gca,'Position');
        pl = pos(1);
        pw = pos(3);
    else
        pos = get(gca,'Position');
        if pos(1) < pl
            pl = pos(1);
        end
        if pos(1)+pos(3)-pl > pw
            pw = pos(1)+pos(3)-pl;
        end
    end

% %     if i == length(kvec)-1
%         pos1 = get(gca,'Position');
%     end
%     if i==length(kvec)
% %     elseif i == length(kvec)
%         pos2 = get(gca,'Position');
%     end

    jj=jj+1;

end

%single colorbar for all plots

% pl = pos1(1);
% pw = pos2(1)+pos2(3)-pl;

% if length(kvec) > 1

    axes('Position', [pl 0.05 pw 0.3], 'Visible', 'off');
    h2=colorbar('location','southoutside','FontSize',fontsize);
    set(h2,'XLim',[minT50 maxT50]);
    set(gca,'CLim',[minT50 maxT50]);

    % tit = 'Agglomerative clustering';
    % 
    supAxes = [0.075 0.075 0.85 0.85];

    [ax,h] = suplabel(titstr,'t',supAxes);
    set(h,'FontSize',1.25*fontsize);
    
% else
%     
%     colorbar('location','southoutside','FontSize',fontsize);
%     title(titstr,'FontSize',1.25*fontsize);
% end