function [pcellranksum,hcellranksum]=ranksumTest(T,M,alpha,xvec,yvec)

% T is a matrix of size numNodes-by-lkvec, where lkvec is the length of the
% kvector used in the clustering algorithm (i.e., the number of different
% clustering trials). Each column represents the outcome of a single
% clustering analysis, and each entry corresponds to that node's assigned
% cluster ID.
%
% The goal here is, for each column of T, to compare all patch-average CDFs
% against each other using the Kolmogorov-Smirnov test to see whether they
% are significantly different from each other.

[~,c] = size(T);

ny = length(yvec);
nx = length(xvec);

pcellranksum = cell(c,1);
hcellranksum = cell(c,1);

Mdsfullvec = M.ds_full(:); 

for i=1:c
    
    tic
    fprintf('ranksumTest: Starting i = %i of %i\n',i,c)
    
    Ttemp = T(:,i);

    
    % get the grains for input to rank sum
    grainscell = cell(max(Ttemp),1); %will contain vectors of raw grain sizes
    
    for aa = 1:max(Ttemp)
        tempgrains = [];
        inds = find(Ttemp == aa);
        for jj=1:length(inds)
           tempgrains = [tempgrains; Mdsfullvec{inds(jj)}];
        end
        grainscell{aa} = log2(tempgrains); %need to be in psi units!

    end
    
    
    
    pValue = zeros(max(Ttemp));
    H = -2 .* ones(max(Ttemp));
    for aa = 1:max(Ttemp)-1
        for bb=aa+1:max(Ttemp)
            
            [p,h] = ranksum(grainscell{aa},grainscell{bb},'alpha',alpha);
            
            pValue(aa,bb) = p;
            H(aa,bb) = -h;

        end
    end
    
    pcellranksum{i} = pValue;
    hcellranksum{i} = H;
    
    
    % plot patches that are the same, if any
    if max(max(H)) == 0
        figure
        [rr,cc] = find(H==0);
        k=1;
        for jj=1:length(rr)
            dum = zeros(ny,nx);
            dum = dum(:);
            inds1 = find(Ttemp == rr(jj));
            inds2 = find(Ttemp == cc(jj));
            dum(inds1) = 1;
            dum(inds2) = 2;
            subplot(length(rr),1,k)
            imagesc(xvec,yvec,reshape(dum,ny,nx));
            axis equal
            set(gca,'YDir','reverse')
            ylim([yvec(1)-(yvec(2)-yvec(1))/2 yvec(ny)+(yvec(ny)-yvec(ny-1))/2])
            xlim([xvec(1)-(xvec(2)-xvec(1))/2 xvec(nx)+(xvec(nx)-xvec(nx-1))/2])
            title(['i = ' num2str(i) ', rr = ' num2str(rr(jj)) ...
                ', cc = ' num2str(cc(jj)) ', pvalue = ' num2str(pValue(rr(jj),cc(jj)))])
            k=k+1;
            
        end
        
        %plot the cdfs; all of them in black, the two of interest in blue
        %and red.
        for jj=1:length(rr)
            figure
            for kk=1:max(Ttemp)
                [f1,xx1] = ecdf(grainscell{kk});
                plot(xx1,f1,'-k')
                hold on
            end
            [f1,xx1] = ecdf(grainscell{rr(jj)});
            [f2,xx2] = ecdf(grainscell{cc(jj)});
            plot(xx1,f1,'-b');
            hold on
            plot(xx2,f2,'-r');
%             legend('rr','cc')
            title(['i = ' num2str(i) ', rr = ' num2str(rr(jj)) ...
                ' (blue), cc = ' num2str(cc(jj)) ' (red), pvalue = ' ...
                num2str(pValue(rr(jj),cc(jj)))])
        end
    end

    toc
    
end