function [u_sort,evals_sort] = spectral_vonlux(W,k)
%
% outputs:
%   u_sort: eigenvectors, sorted by their eigenvalues
%   evals_sort: sorted eigenvalues (ascending)
%   k: number of eigenvectors to compute
%
% inputs:
%   W: similarity matrix


[r,c] = size(W);
numNodes = r;

%% construct degree matrix D (vectorized)
%
% D is a sparse matrix where the diagonal elements are the sum of the edge
% weights of each node. This is easily computed from the W matrix by
% summing down the columns or across the rows.

d = sum(W,1)';% + sum(W,2);
D = spdiags(d,0,numNodes,numNodes);

%% construct Laplacian matrix L
%
% There are three types of Laplacians presented in von Luxburg.
% 1) unnormalized: L = D - W;
% 2) normalized symmetric: L_sym = I - D^(-1/2)*W*D^(-1/2)
% 3) normalized random walk: L_rw = I - D^(-1)*W
%
% Ncut uses the unnormalized Laplacian L. von Luxburg reccomends using
% normalized Laplacians, and L_rw instead of L_sym. However, he says that
% the degree distribution of the similarity graph is important and "if the
% graph is very regular and most vertices have approximately the same
% degree, then all the Laplacians are very similar to each other, and will
% work equally well for clustering."

L = D - W;

%% compute the first k eigenvectors

opts.disp = 0;

% [u2,eigenvalues2] = eig(full(L),full(D));
[u,eigenvalues] = eigs(L,D,k,'sm',opts); %normalized
% [u,eigenvalues] = eigs(L,k,'sm'); %unnormalized

[evals_sort, ix] = sort(diag(eigenvalues));
u_sort = u(:,ix);


